'' =================================================================================================
''
''   File....... jm_hc-06_configurator.spin
''   Purpose.... Configuration utility for HC-06 module
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 18 FEB 2015 
''
'' =================================================================================================

{{

  Source: http://wiki.pinguino.cc/index.php/SPP_Bluetooth_Modules

  HC-06 AT commands
  
   * AT commands can only be sent to the device using a wired, not wireless, connection.
  
   * The bluetooth device will only respond to AT commands before it is paired/connected.
  
   * AT commands do not need a terminating CR/LF sequence.
  
   * AT commands must be entered quickly (cut and paste or use macro).

   
  The HC-06 firmware has a very limited subset of AT commands as listed below:
  
   * AT responds OK

   * AT+BAUDn where n = 1..C
     1=1200, 2=2400, 3=4800, 4=9600 (default), 5=19200, 6=38400, 7=57600, 8=115200, 9=230400,
     A=460800, B=921600, C=1382400 
     -- Responds OKnnnn. Retained across power offs.
     
   * AT+NAMEname where name 20 or fewer characters. Responds OKname. Retained across power offs.
   
   * AT+PINnnnn sets the pairing password. Responds OKsetpin.
   
   * AT+Pn where n is N (no parity), O (odd parity) or E (even parity). Firmware version higher than Linvor 1.5 only.
   
   * AT+VERSION responds with the firmware version.

}}


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' use 5MHz crystal

  CLK_FREQ = _xinfreq * (_clkmode >> 6)                          ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                    ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                                ' ticks in 1us
                                                                  
                                                                  
con { activity board io pins }                                    
                                                                  
  RX1     = 31                                                   ' programming / terminal
  TX1     = 30                                                    
                                                                  
  SDA     = 29                                                   ' eeprom / i2c
  SCL     = 28                                                    
                                                                  
  AUD_RT  = 27                                                   ' audio (d/a) outputs
  AUD_LF  = 26                                                    
                                                                  
  LED_1   = 27                                                   ' leds (on audio outputs)
  LED_L   = 26                                                    
                                                                  
  SD_CS   = 25                                                   ' usd card
  SD_DI   = 24                                                    
  SD_CLK  = 23                                                    
  SD_DO   = 22                                                    
                                                                  
  ADC_CS  = 21                                                   ' adc input
  ADC_CLK = 20                                                    
  ADC_DO  = 19 { miso }                                           
  ADC_DI  = 18 { mosi }                                           
                                                                  
  SVO_6   = 17                                                   ' servo headers
  SVO_5   = 16                                                    
                                                                  
  SVO_4   = 15                                                    
  SVO_3   = 14                                                    
                                                                  
  SVO_2   = 13                                                    
  SVO_1   = 12                                                    

  RX2     = 11                                                   ' <-- bluetooth tx (3.3v mode)
  TX2     = 10                                                   ' --> bluetooth rx


con

  TRM_BAUD = 115_200                                             ' terminal baud setting
  BT_BAUD  =   9_600                                             ' starting bluetooth baud setting

  BUF_SIZE = 40


obj

  term   : "jm_fullduplexserial"                                 ' * serial for terminal
  btio   : "jm_fullduplexserial"                                 ' * serial for bluetooth
  parser : "jm_parser"                                           '   parsing engine
                                                         
' * uses cog when loaded                                           
                                                                  
                                                                  
var

  byte  buffer[BUF_SIZE]                                         ' string buffer for bt commands


dat { valid characters and tokens }

  CHAR_SET              byte    "-_0123456789"                   ' valid characters for this token set
                        byte    "ABCDEFGHIJKLMNOPQRSTUVWXYZ", 0   
  
  TOKEN_LIST                                                     ' tokens used by this program
  TOKEN_AT              byte    "AT", 0
  TOKEN_BAUD            byte    "BAUD", 0    
  TOKEN_NAME            byte    "NAME", 0    
  TOKEN_PIN             byte    "PIN", 0
  TOKEN_VERSION         byte    "VERSION", 0
  TOKEN_SLAVE           byte    "SLAVE", 0
  TOKEN_MASTER          byte    "MASTER", 0
  TOKEN_HELP            byte    "HELP", 0    

  
con { enumerated token indexes }

  #0, T_AT, T_BAUD, T_NAME, T_PIN, T_VERSION, T_SLAVE, T_MASTER, T_HELP

  TOKEN_COUNT = T_HELP+1                                         ' total # of known tokens

                                                                          
pub main | c                                                    
                                                                  
  setup

  term.tx(term#CLS)
  term.str(string("HC-06 Configurator", 13))

  term.rxflush                                                   ' clear any start-up trash
  btio.rxflush

  repeat
    c := term.rx                                                 ' get char from terminal
    if (parser.enqueue(c))                                       ' send to parser, if tokens ready
      process_cmd                                                '  process valid tokens
      parser.reset                                               '  reset parser for next command


pub process_cmd | tidx

  tidx := parser.get_token_id(parser.token_addr(0))              ' get index of command token
  
  case tidx                                                      ' handle valid tokens
    T_AT      : ping
    T_BAUD    : set_baud
    T_NAME    : set_name
    T_PIN     : set_pin
    T_VERSION : show_version
    T_SLAVE   : set_slave
    T_MASTER  : set_master                                       ' requires HC-06 version 1.7 firmware
    T_HELP    : show_help
    other     : invalid_cmd


pub ping

'' Verifies HC-06 connected at correct baud rate
'' -- HC-06 responds with "OK"

  if (parser.token_count <> 1)                                   ' check token count
    term.str(@UseHdr)
    term.str(@ATMsg)
    return

  set_and_get(string("AT"))

        
pub set_baud | check, baud

'' Sets new baud rate into HC-06
'' -- limited to FDS-friendly baud rates
'' -- restarts serial connection to bt module at new baud rate

  if (parser.token_count <> 2)                                   ' check token count
    term.str(@UseHdr)
    term.str(@BaudMsg)
    return

  check := parser.token_is_number(1)

  if (check)
    baud := parser.token_value(1)   
    check := lookdown(baud : 1200, 2400, 4800, 9600, 19200, 38400, 57600, 115200)

  if (check == false)
    term.str(string("Invalid baud setting", 13))
    return

  ' set new baud in module and connection

  bytefill(@buffer, 0, BUF_SIZE)                                 ' clear bluetooth buffer
  bytemove(@buffer, string("AT+BAUD"), 7)                        ' set command string
  buffer[7] := check + "0"                                       ' add parameter

  set_and_get(@buffer)                                           ' update module

  btio.start(RX2, TX2, %0000, baud)                              ' reset serial link 


pub set_name | p_name, len

'' Sets name in HC-06
'' -- note that parser changes everything to uppercase

  if (parser.token_count <> 2)                                   ' check token count
    term.str(@UseHdr)
    term.str(@NameMsg)
    return

  p_name := parser.token_addr(1)                                 ' get pointer to name
  len := strsize(p_name)

  bytefill(@buffer, 0, BUF_SIZE)                                 ' clear bluetooth buffer
  bytemove(@buffer, string("AT+NAME"), 7)                        ' set command string
  bytemove(@buffer+7, p_name, len)                               ' add name
  
  set_and_get(@buffer)                                           ' update module

        
pub set_pin | p_pin, len

'' Set PIN (pairing passcode) in HC-06

  if (parser.token_count <> 2)                                   ' check token count
    term.str(@UseHdr)
    term.str(@PinMsg)
    return

  p_pin := parser.token_addr(1)
  len := strsize(p_pin)

  bytefill(@buffer, 0, BUF_SIZE)                                 ' clear bluetooth buffer
  bytemove(@buffer, string("AT+PIN"), 6)                         ' set command string
  bytemove(@buffer+6, p_pin, len)                                ' add pin
  
  set_and_get(@buffer)                                           ' update module


pub show_version

'' Ask HC-06 firmware version
'' -- response something like "OKlinvorV1.8"

  if (parser.token_count <> 1)                                   ' check token count
    term.str(@UseHdr)
    term.str(@VersionMsg)
    return

  set_and_get(string("AT+VERSION"))


pub set_slave

'' Sets HC-06 to slave mode
'' -- default configuration for HC-06

  if (parser.token_count <> 1)                                   ' check token count
    term.str(@UseHdr)
    term.str(@SlaveMsg)
    return

  set_and_get(string("AT+ROLE=S"))


pub set_master | k                                               ' NOT RECOMMENDED -- NO CONTROL OVER PAIRING

'' Sets HC-06 to master mode

  if (parser.token_count <> 1)                                   ' check token count
    term.str(@UseHdr)
    term.str(@MasterMsg)
    return

  term.tx(term#CR)
  term.str(string("Are you sure you want to set MASTER mode? (Y/N) "))
  term.rxflush
  k := term.rx
  ifnot ((k == "Y") or (k == "y"))
    return

  set_and_get(string("AT+ROLE=M"))

    
dat { help strings }

  UseHdr                byte    "Usage: ", 0

  ATMsg                 byte    "AT <CR>", 13, 0
  BaudMsg               byte    "BAUD newbaud <CR>", 13, 0
  NameMsg               byte    "NAME newname <CR>", 13, 0
  PinMsg                byte    "PIN newcode <CR>", 13, 0
  VersionMsg            byte    "VERSION <CR>", 13, 0
  SlaveMsg              byte    "SLAVE <CR>", 13, 0  
  MasterMsg             byte    "MASTER <CR>", 13, 0             ' not recommended! 

    
pub show_help

'' List available commands

  term.tx(13)
  term.str(@ATMsg)
  term.str(@BaudMsg)
  term.str(@NameMsg)
  term.str(@PinMsg)
  term.str(@VersionMsg)   
  
  
pub invalid_cmd

  term.str(string(13, "Invalid command: "))
  term.str(parser.queue_addr)                                    ' show me what was bogus
  term.tx(13)


pub set_and_get(p_str)

'' Send string at p_str to HC-06 and wait for response

  term.tx(13)
  term.str(p_str)
  term.tx(13)
  
  btio.str(p_str) 
  get_bt


pub get_bt | c

'' RX and display chars from HC-06 module

  repeat
    c := btio.rxtime(1000)                                       ' wait up to 1s
    if (c => 0)                                                  ' good char
      term.tx(c)                                                 ' print to terminal
    else
      quit                                                       ' no char, timeout
    
  term.tx(13)

                             
pub setup                                                         
                                                                  
  term.start(RX1, TX1, %0000, TRM_BAUD)                          ' start serial for terminal

  btio.start(RX2, TX2, %0000, BT_BAUD)                           ' start serial for bluetooth

  parser.start(@CHAR_SET, @TOKEN_LIST, TOKEN_COUNT, false)       ' setup parsing engine 
                                                                  
  pause(2000)                                                    ' let everything get ready
                                                                  

con

  { ------------- }
  {  B A S I C S  }
  { ------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                    ' delay must be > 0
    return
  else
    t := cnt - 1776                                              ' sync with system counter
    repeat ms                                                    ' run delay
      waitcnt(t += MS_001)


pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]

                                                                  
dat { license }                                                   
                                                                  
{{                                                                

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}